import { Service } from "../service"
import * as express from "express"
import * as bodyParser from "body-parser"
import { MongodbService } from "./mongodbService"
import * as http from "http"
export class ExpressService<TConfig extends {
    http?: {
        port?: number
        host?: string
        offline?: boolean
    }
    staticResources?: {
        path: string
        route: string
    }[]
} = any> extends Service<TConfig> {
    name = "ExpressService" as "ExpressService"
    dependencies = ["MongodbService"]
    services: {
        MongodbService: MongodbService
    }
    server: express.Application
    constructor(config: TConfig) {
        super()
        let http = config.http || {}
        this.port = http.port || 8080
        this.host = http.host || "0.0.0.0"
        this.offline = http.offline
        let staticResources = config.staticResources || []
        for (let res of staticResources) {
            this.static(res.path, res.route)
        }
    }
    public offline: boolean
    private host: string
    private port: number
    private httpServer: http.Server
    initialize(done) {
        this.server = express()
        for (let info of this.staticPathes) {
            this.useStatic(info)
        }
        this.server.use(bodyParser.json({ limit: "2mb" })); // for parsing application/json
        this.server.use(bodyParser.urlencoded({ extended: true, limit: "2mb" })); // for parsing application/x-www-form-urlencoded
        if (!this.offline) {
            this.initHttp(() => {
                done()
            })
        } else {
            done()
        }
    }
    public initHttp(callback: Function) {
        this.httpServer = this.server.listen(parseInt(this.port.toString()), this.host, () => {
            console.log("Express listen at", `${this.host}:${this.port}`)
            callback()
        })
    }
    asBuiltIn() {
        console.log(`ExpressService default http://${this.host}:${this.port}/`)
    }
    private staticPathes: {
        route?: string
        path: string
    }[] = []
    static(path: string, route?: string) {
        this.staticPathes.push({
            path, route
        })
        if (this.server) {
            this.useStatic(this.staticPathes[this.staticPathes.length - 1])
        }
    }
    private useStatic(info: { route?: string, path: string }) {
        if (!info.path) {
            console.error(info)
            console.error("Static resource requires a path")
            return
        }
        if (info.route) {
            this.server.use(info.route, express.static(info.path))
        } else {
            this.server.use(express.static(info.path))
        }
    }
}
