"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var route_1 = require("../../route");
var Leaf = require("leaf-ts");
exports.Errors = Leaf.ErrorDoc.create()
    .define("Forbidden")
    .define("AuthorizationFailed")
    .generate();
var LegacyUserAPIService = /** @class */ (function (_super) {
    __extends(LegacyUserAPIService, _super);
    function LegacyUserAPIService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.name = "api.legacyUser";
        _this.dependencies = ["ExpressService", "RouteSessionService", "LegacyUserService", "LegacyUserDefinitionService"];
        _this.prefix = "/api/";
        return _this;
    }
    LegacyUserAPIService.prototype.initialize = function () {
        this.addRoutes(module.exports);
        this.installTo(this.services.ExpressService.server);
    };
    return LegacyUserAPIService;
}(route_1.RouteService));
exports.LegacyUserAPIService = LegacyUserAPIService;
var SessionLegacyUserProvider = /** @class */ (function (_super) {
    __extends(SessionLegacyUserProvider, _super);
    function SessionLegacyUserProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SessionLegacyUserProvider.prototype.handle = function () {
        var _this = this;
        var session = this.req.session;
        if (!session) {
            this.next();
        }
        var name = this.req.session.legacyUsername;
        if (!name) {
            this.next();
            return;
        }
        this.services.LegacyUserService.getLegacyUserByLegacyUsername(name, function (err, legacyUser) {
            if (err) {
                _this.next();
                return;
            }
            _this.info.legacyUser = legacyUser;
            _this.next();
        });
    };
    return SessionLegacyUserProvider;
}(route_1.MiddlewareRoute));
exports.SessionLegacyUserProvider = SessionLegacyUserProvider;
var Register = /** @class */ (function (_super) {
    __extends(Register, _super);
    function Register() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.method = route_1.Method.POST;
        _this.path = "legacyUser";
        _this.minLegacyUsernameLength = 3;
        _this.maxLegacyUsernameLength = 32;
        _this.minPasswordLength = 8;
        _this.maxPasswordLength = 1024;
        return _this;
    }
    Register.prototype.checkLegacyUserValidation = function (legacyUser) {
        return this.services.LegacyUserDefinitionService.LegacyUserValidator.check(legacyUser);
    };
    Register.prototype.handle = function () {
        var _this = this;
        var legacyUserToCheck = this.body;
        try {
            this.checkLegacyUserValidation(legacyUserToCheck);
        }
        catch (err) {
            this.error(err);
            return;
        }
        this.services.LegacyUserService.createLegacyUser(legacyUserToCheck, function (err, legacyUser) {
            if (err) {
                _this.error(err);
                return;
            }
            _this.req.session.legacyUsername = legacyUser.legacyUsername;
            _this.success(null);
        });
    };
    return Register;
}(route_1.APIRoute));
exports.Register = Register;
var GetLegacyUserProfile = /** @class */ (function (_super) {
    __extends(GetLegacyUserProfile, _super);
    function GetLegacyUserProfile() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.method = route_1.Method.GET;
        _this.path = "legacyUser";
        return _this;
    }
    GetLegacyUserProfile.prototype.handle = function () {
        if (!this.info.legacyUser) {
            this.error(new exports.Errors.Forbidden);
            return;
        }
        var legacyUser = this.info.legacyUser;
        this.success({
            legacyUsername: legacyUser.legacyUsername,
            email: legacyUser.email,
            displayName: legacyUser.displayName
        });
    };
    return GetLegacyUserProfile;
}(route_1.APIRoute));
exports.GetLegacyUserProfile = GetLegacyUserProfile;
var Login = /** @class */ (function (_super) {
    __extends(Login, _super);
    function Login() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.method = route_1.Method.PUT;
        _this.path = "legacyUser";
        return _this;
    }
    Login.prototype.handle = function () {
        var _this = this;
        var loginLegacyUser = this.body;
        this.services.LegacyUserService.getLegacyUserByLogin(loginLegacyUser, function (err, legacyUser) {
            if (err) {
                _this.error(err);
                return;
            }
            if (!legacyUser) {
                _this.error(new exports.Errors.AuthorizationFailed);
                return;
            }
            _this.req.session.legacyUsername = legacyUser.legacyUsername;
            _this.success(legacyUser);
        });
    };
    return Login;
}(route_1.APIRoute));
exports.Login = Login;
var Logout = /** @class */ (function (_super) {
    __extends(Logout, _super);
    function Logout() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.method = route_1.Method.DELETE;
        _this.path = "legacyUser";
        return _this;
    }
    Logout.prototype.handle = function () {
        this.req.session.legacyUsername = null;
        this.success(null);
    };
    return Logout;
}(route_1.APIRoute));
exports.Logout = Logout;
//# sourceMappingURL=api.user.js.map